'use strict';
var path = require('path');
var gutil = require('gulp-util');
var through = require('through2');
var objectAssign = require('object-assign');
var file = require('vinyl-file');
var sortKeys = require('sort-keys');

var regFiles = /([^\"\']*\.(js|css|png|jpg|gif))/gm;


function getManifestFile(opts, cb) {
	file.read(opts.path, opts, function (err, manifest) {
		if (err) {
			// not found
			if (err.code === 'ENOENT') {
				cb(null, new gutil.File(opts));
			} else {
				cb(err);
			}

			return;
		}

		cb(null, manifest);
	});
}

var plugin = function (pth, opts) {
	opts = objectAssign({
		path: 'rev-mapping.json',
		merge: true,
		// Apply the default JSON transformer.
		// The user can pass in his on transformer if he wants. The only requirement is that it should
		// support 'parse' and 'stringify' methods.
		transformer: JSON
	}, opts, pth);

	var manifest = {};
	return through.obj(function (file, enc, cb) {
		if (file.isNull()) {
			cb(null, file);
			return;
		}

		if (file.isStream()) {
			cb(new gutil.PluginError('gulp-rev-mapping', 'Streaming not supported'));
			return;
		}

		file.relPath = path.relative(process.cwd(), file.path).replace(/\\/g, '/');
		file.fileList = file.contents.toString().match(regFiles);

		getManifestFile(opts, function (err, manifestFile) {
			if (err) {
				cb(err);
				return;
			}

			if (opts.merge && !manifestFile.isNull()) {
				var oldManifest = {};

				try {
					oldManifest = opts.transformer.parse(manifestFile.contents.toString());
				} catch (err) {}

				manifest = objectAssign(oldManifest, manifest);
			}

			if(file.fileList) {
				file.fileList.forEach(function(filename) {
					filename = path.basename(filename);
					if(!manifest[filename]) {
						manifest[filename] = [];
					}
					if(manifest[filename].indexOf(file.relPath) == -1) {
						manifest[filename].push(file.relPath);
					}
				});

				manifestFile.contents = new Buffer(opts.transformer.stringify(sortKeys(manifest), null, '  '));
				this.push(manifestFile);
			}

			cb();
		}.bind(this));
	});
};

module.exports = plugin;
